@@ -161,6 +161,42 @@ update_default_input (GvcMixerDialog *dialog)
         } while (gtk_tree_model_iter_next (model, &iter));
 }
 
+static void
+update_description (GvcMixerDialog *dialog,
+                    guint column,
+                    const char *value,
+                    GvcMixerStream *stream)
+{
+        GtkTreeModel *model;
+        GtkTreeIter   iter;
+        guint         id;
+
+        if (GVC_IS_MIXER_SOURCE (stream))
+                model = gtk_tree_view_get_model (GTK_TREE_VIEW (dialog->priv->input_treeview));
+        else if (GVC_IS_MIXER_SINK (stream))
+                model = gtk_tree_view_get_model (GTK_TREE_VIEW (dialog->priv->output_treeview));
+        else
+                g_assert_not_reached ();
+        gtk_tree_model_get_iter_first (model, &iter);
+
+        id = gvc_mixer_stream_get_id (stream);
+        do {
+                guint       current_id;
+
+                gtk_tree_model_get (model, &iter,
+                                    ID_COLUMN, &current_id,
+                                    -1);
+                if (id != current_id)
+                        continue;
+
+                gtk_list_store_set (GTK_LIST_STORE (model),
+                                    &iter,
+                                    column, value,
+                                    -1);
+                break;
+        } while (gtk_tree_model_iter_next (model, &iter));
+}
+
 static void
 port_selection_changed (GvcComboBox *combo_box,
                         const char  *port,
@@ -232,7 +268,7 @@ update_output_settings (GvcMixerDialog *dialog)
         }
         gtk_box_pack_start (GTK_BOX (dialog->priv->output_settings_box),
                             dialog->priv->output_balance_bar,
-                            FALSE, FALSE, 12);
+                            FALSE, FALSE, 6);
         gtk_widget_show (dialog->priv->output_balance_bar);
 
         if (gvc_channel_map_can_fade (map)) {
@@ -244,7 +280,7 @@ update_output_settings (GvcMixerDialog *dialog)
                 }
                 gtk_box_pack_start (GTK_BOX (dialog->priv->output_settings_box),
                                     dialog->priv->output_fade_bar,
-                                    FALSE, FALSE, 12);
+                                    FALSE, FALSE, 6);
                 gtk_widget_show (dialog->priv->output_fade_bar);
         }
 
@@ -257,7 +293,7 @@ update_output_settings (GvcMixerDialog *dialog)
                 }
                 gtk_box_pack_start (GTK_BOX (dialog->priv->output_settings_box),
                                     dialog->priv->output_lfe_bar,
-                                    FALSE, FALSE, 12);
+                                    FALSE, FALSE, 6);
                 gtk_widget_show (dialog->priv->output_lfe_bar);
         }
 
@@ -276,7 +312,10 @@ update_output_settings (GvcMixerDialog *dialog)
 
                 gtk_box_pack_start (GTK_BOX (dialog->priv->output_settings_box),
                                     dialog->priv->output_port_combo,
-                                    TRUE, FALSE, 0);
+                                    TRUE, FALSE, 6);
+
+                gvc_combo_box_set_size_group (GVC_COMBO_BOX (dialog->priv->output_port_combo), dialog->priv->size_group, FALSE);
+
                 gtk_widget_show (dialog->priv->output_port_combo);
         }
 
@@ -530,6 +569,7 @@ update_input_settings (GvcMixerDialog *dialog)
                 g_signal_connect (G_OBJECT (dialog->priv->input_port_combo), "changed",
                                   G_CALLBACK (port_selection_changed), dialog);
 
+                gvc_combo_box_set_size_group (GVC_COMBO_BOX (dialog->priv->input_port_combo), dialog->priv->size_group, FALSE);
                 gtk_box_pack_start (GTK_BOX (dialog->priv->input_settings_box),
                                     dialog->priv->input_port_combo,
                                     TRUE, TRUE, 0);
@@ -651,7 +691,7 @@ on_adjustment_value_changed (GtkAdjustment  *adjustment,
 
         stream = g_object_get_data (G_OBJECT (adjustment), "gvc-mixer-dialog-stream");
         if (stream != NULL) {
-        	GObject *bar;
+                GObject *bar;
                 gdouble volume, rounded;
                 char *name;
 
@@ -735,6 +775,16 @@ lookup_combo_box_for_stream (GvcMixerDialog *dialog,
         return NULL;
 }
 
+static void
+on_stream_description_notify (GvcMixerStream *stream,
+                              GParamSpec     *pspec,
+                              GvcMixerDialog *dialog)
+{
+        update_description (dialog, NAME_COLUMN,
+                            gvc_mixer_stream_get_description (stream),
+                            stream);
+}
+
 static void
 on_stream_port_notify (GObject        *object,
                        GParamSpec     *pspec,
@@ -1001,6 +1051,10 @@ add_stream (GvcMixerDialog *dialog,
                                     ACTIVE_COLUMN, is_default,
                                     ID_COLUMN, gvc_mixer_stream_get_id (stream),
                                     -1);
+                g_signal_connect (stream,
+                                  "notify::description",
+                                  G_CALLBACK (on_stream_description_notify),
+                                  dialog);
         } else if (GVC_IS_MIXER_SINK (stream)) {
                 GtkTreeModel  *model;
                 GtkTreeIter    iter;
@@ -1017,6 +1071,10 @@ add_stream (GvcMixerDialog *dialog,
                                     ID_COLUMN, gvc_mixer_stream_get_id (stream),
                                     SPEAKERS_COLUMN, gvc_channel_map_get_mapping (map),
                                     -1);
+                g_signal_connect (stream,
+                                  "notify::description",
+                                  G_CALLBACK (on_stream_description_notify),
+                                  dialog);
         }
 
         if (bar != NULL) {
@@ -1439,7 +1497,7 @@ on_card_selection_changed (GtkTreeSelection *selection,
 
         gtk_box_pack_start (GTK_BOX (dialog->priv->hw_settings_box),
                             dialog->priv->hw_profile_combo,
-                            FALSE, FALSE, 12);
+                            TRUE, TRUE, 6);
         gtk_widget_show (dialog->priv->hw_profile_combo);
 
         g_object_set_data (G_OBJECT (dialog->priv->hw_profile_combo), "card", card);
@@ -1576,13 +1634,17 @@ gvc_mixer_dialog_constructor (GType                  type,
         gtk_dialog_add_button (GTK_DIALOG (self), "gtk-close", GTK_RESPONSE_OK);
 
         main_vbox = gtk_dialog_get_content_area (GTK_DIALOG (self));
+        gtk_box_set_spacing (GTK_BOX (main_vbox), 2);
 
         gtk_container_set_border_width (GTK_CONTAINER (self), 6);
 
         self->priv->output_stream_box = gtk_hbox_new (FALSE, 12);
+        alignment = gtk_alignment_new (0, 0, 1, 1);
+        gtk_alignment_set_padding (GTK_ALIGNMENT (alignment), 12, 0, 0, 0);
+        gtk_container_add (GTK_CONTAINER (alignment), self->priv->output_stream_box);
         gtk_box_pack_start (GTK_BOX (main_vbox),
-                            self->priv->output_stream_box,
-                            FALSE, FALSE, 12);
+                            alignment,
+                            FALSE, FALSE, 0);
         self->priv->output_bar = create_bar (self, self->priv->size_group, TRUE);
         gvc_channel_bar_set_name (GVC_CHANNEL_BAR (self->priv->output_bar),
                                   _("_Output volume: "));
@@ -1593,7 +1655,8 @@ gvc_mixer_dialog_constructor (GType                  type,
         self->priv->notebook = gtk_notebook_new ();
         gtk_box_pack_start (GTK_BOX (main_vbox),
                             self->priv->notebook,
-                            TRUE, TRUE, 6);
+                            TRUE, TRUE, 0);
+        gtk_container_set_border_width (GTK_CONTAINER (self->priv->notebook), 5);
 
         /* Set up accels (borrowed from Empathy) */
         accel_group = gtk_accel_group_new ();
@@ -1625,7 +1688,7 @@ gvc_mixer_dialog_constructor (GType                  type,
                                   _("_Alert volume: "));
         gtk_widget_set_sensitive (self->priv->effects_bar, FALSE);
         gtk_box_pack_start (GTK_BOX (self->priv->sound_effects_box),
-                            self->priv->effects_bar, FALSE, FALSE, 12);
+                            self->priv->effects_bar, FALSE, FALSE, 0);
 
         self->priv->sound_theme_chooser = gvc_sound_theme_chooser_new ();
         gtk_box_pack_start (GTK_BOX (self->priv->sound_effects_box),
@@ -1691,14 +1754,17 @@ gvc_mixer_dialog_constructor (GType                  type,
         gvc_channel_bar_set_high_icon_name (GVC_CHANNEL_BAR (self->priv->input_bar),
                                             "audio-input-microphone-high");
         gtk_widget_set_sensitive (self->priv->input_bar, FALSE);
+        alignment = gtk_alignment_new (0, 0, 1, 1);
+        gtk_alignment_set_padding (GTK_ALIGNMENT (alignment), 6, 0, 0, 0);
+        gtk_container_add (GTK_CONTAINER (alignment), self->priv->input_bar);
         gtk_box_pack_start (GTK_BOX (self->priv->input_box),
-                            self->priv->input_bar,
-                            FALSE, FALSE, 12);
+                            alignment,
+                            FALSE, FALSE, 0);
 
         box = gtk_hbox_new (FALSE, 6);
         gtk_box_pack_start (GTK_BOX (self->priv->input_box),
                             box,
-                            FALSE, FALSE, 12);
+                            FALSE, FALSE, 6);
 
         sbox = gtk_hbox_new (FALSE, 6);
         gtk_box_pack_start (GTK_BOX (box),
@@ -1718,8 +1784,7 @@ gvc_mixer_dialog_constructor (GType                  type,
                                  GVC_LEVEL_SCALE_LINEAR);
         gtk_box_pack_start (GTK_BOX (box),
                             self->priv->input_level_bar,
-                            TRUE, TRUE, 0);
-        gtk_widget_show (self->priv->input_level_bar);
+                            TRUE, TRUE, 6);
 
         ebox = gtk_hbox_new (FALSE, 6);
         gtk_box_pack_start (GTK_BOX (box),
@@ -1798,8 +1863,8 @@ gvc_mixer_dialog_constructor (GType                  type,
         label = gtk_frame_get_label_widget (GTK_FRAME (box));
         _gtk_label_make_bold (GTK_LABEL (label));
         gtk_frame_set_shadow_type (GTK_FRAME (box), GTK_SHADOW_NONE);
-        gtk_box_pack_start (GTK_BOX (self->priv->output_box), box, TRUE, TRUE, 12);
-        self->priv->output_settings_box = gtk_vbox_new (FALSE, 12);
+        gtk_box_pack_start (GTK_BOX (self->priv->output_box), box, FALSE, FALSE, 12);
+        self->priv->output_settings_box = gtk_vbox_new (FALSE, 0);
         gtk_container_add (GTK_CONTAINER (box), self->priv->output_settings_box);
 
         /* Applications */
@@ -1831,7 +1896,7 @@ gvc_mixer_dialog_constructor (GType                  type,
                           G_CALLBACK (on_control_card_removed),
                           self);
 
-        gtk_widget_show_all (GTK_WIDGET (self));
+        gtk_widget_show_all (main_vbox);
 
         streams = gvc_mixer_control_get_streams (self->priv->mixer_control);
         for (l = streams; l != NULL; l = l->next) {
@@ -1940,22 +2005,38 @@ gvc_mixer_dialog_new (GvcMixerControl *control)
         return GVC_MIXER_DIALOG (dialog);
 }
 
+enum {
+        PAGE_EVENTS,
+        PAGE_HARDWARE,
+        PAGE_INPUT,
+        PAGE_OUTPUT,
+        PAGE_APPLICATIONS
+};
+
 gboolean
 gvc_mixer_dialog_set_page (GvcMixerDialog *self,
                            const char     *page)
 {
+        guint num;
+
         g_return_val_if_fail (self != NULL, FALSE);
 
-        if (g_ascii_strncasecmp(page, "playback",8) == 0)
-                gtk_notebook_set_current_page (GTK_NOTEBOOK (self->priv->notebook), 2);
-        else if (g_ascii_strncasecmp(page, "recording",9) == 0)
-                gtk_notebook_set_current_page (GTK_NOTEBOOK (self->priv->notebook), 1);
-        else if (g_ascii_strncasecmp(page, "effects",7) == 0)
-                gtk_notebook_set_current_page (GTK_NOTEBOOK (self->priv->notebook), 0);
-        else if (g_ascii_strncasecmp(page, "applications",12) == 0)
-                gtk_notebook_set_current_page (GTK_NOTEBOOK (self->priv->notebook), 3);
-        else /* default is "playback" */
-                gtk_notebook_set_current_page (GTK_NOTEBOOK (self->priv->notebook), 0);
+        if (page == NULL)
+                num = 0;
+        else if (g_str_equal (page, "effects"))
+                num = PAGE_EVENTS;
+        else if (g_str_equal (page, "hardware"))
+                num = PAGE_HARDWARE;
+        else if (g_str_equal (page, "input"))
+                num = PAGE_INPUT;
+        else if (g_str_equal (page, "output"))
+                num = PAGE_OUTPUT;
+        else if (g_str_equal (page, "applications"))
+                num = PAGE_APPLICATIONS;
+        else
+                num = 0;
+
+        gtk_notebook_set_current_page (GTK_NOTEBOOK (self->priv->notebook), num);
 
         return TRUE;
 }
